"""
Tests for CLI main functionality.
"""

import pytest
import os
import tempfile
import json
from hbat.cli.main import (
    create_parser, 
    load_parameters_from_args, 
    resolve_preset_path,
    load_preset_file,
    list_available_presets,
    get_example_presets_directory
)
from hbat.core.analysis import AnalysisParameters


class TestCLIArgumentParsing:
    """Test CLI argument parsing functionality."""
    
    def test_parser_creation(self):
        """Test that parser can be created."""
        parser = create_parser()
        assert parser is not None
        
        help_text = parser.format_help()
        assert "HBAT" in help_text
        assert "input" in help_text
        assert "--hb-distance" in help_text
    
    def test_basic_argument_parsing(self):
        """Test basic argument parsing."""
        parser = create_parser()
        
        # Test with minimal arguments
        args = parser.parse_args(["test.pdb"])
        assert args.input == "test.pdb"
        
        # Test with output options
        args = parser.parse_args(["test.pdb", "-o", "output.txt"])
        assert args.input == "test.pdb"
        assert args.output == "output.txt"
    
    def test_parameter_arguments(self):
        """Test parameter-specific arguments."""
        parser = create_parser()
        
        args = parser.parse_args([
            "test.pdb",
            "--hb-distance", "3.0",
            "--hb-angle", "130",
            "--mode", "local"
        ])
        
        assert args.hb_distance == 3.0
        assert args.hb_angle == 130.0
        assert args.mode == "local"
    
    def test_preset_arguments(self):
        """Test preset-related arguments."""
        parser = create_parser()
        
        # Test preset option
        args = parser.parse_args(["test.pdb", "--preset", "high_resolution"])
        assert args.preset == "high_resolution"
        
        # Test list presets option
        args = parser.parse_args(["--list-presets"])
        assert args.list_presets is True
        assert args.input is None  # Should be optional when listing presets
    
    def test_output_format_arguments(self):
        """Test output format arguments."""
        parser = create_parser()
        
        args = parser.parse_args([
            "test.pdb",
            "--json", "output.json",
            "--csv", "output.csv",
            "--verbose"
        ])
        
        assert args.json == "output.json"
        assert args.csv == "output.csv"
        assert args.verbose is True
    
    def test_analysis_filter_arguments(self):
        """Test analysis filter arguments."""
        parser = create_parser()
        
        args = parser.parse_args([
            "test.pdb",
            "--no-hydrogen-bonds",
            "--no-halogen-bonds"
        ])
        
        assert args.no_hydrogen_bonds is True
        assert args.no_halogen_bonds is True


class TestParameterLoading:
    """Test parameter loading from CLI arguments."""
    
    def test_default_parameter_loading(self):
        """Test loading default parameters."""
        parser = create_parser()
        args = parser.parse_args(["test.pdb"])
        
        params = load_parameters_from_args(args)
        assert isinstance(params, AnalysisParameters)
        assert params.hb_distance_cutoff > 0
        assert params.hb_angle_cutoff > 0
    
    def test_custom_parameter_loading(self):
        """Test loading custom parameters."""
        parser = create_parser()
        args = parser.parse_args([
            "test.pdb",
            "--hb-distance", "3.2",
            "--hb-angle", "140",
            "--mode", "local"
        ])
        
        params = load_parameters_from_args(args)
        assert params.hb_distance_cutoff == 3.2
        assert params.hb_angle_cutoff == 140.0
        assert params.analysis_mode == "local"
    
    def test_preset_parameter_loading(self):
        """Test loading parameters from preset."""
        parser = create_parser()
        
        # Find an available preset
        presets_dir = get_example_presets_directory()
        if os.path.exists(presets_dir):
            preset_files = [f for f in os.listdir(presets_dir) if f.endswith('.hbat')]
            if preset_files:
                preset_name = preset_files[0].replace('.hbat', '')
                
                args = parser.parse_args(["test.pdb", "--preset", preset_name])
                params = load_parameters_from_args(args)
                
                assert isinstance(params, AnalysisParameters)
                # Parameters should be loaded from preset
                assert params.hb_distance_cutoff > 0
    
    def test_preset_with_override(self):
        """Test preset loading with parameter overrides."""
        parser = create_parser()
        
        # Find an available preset
        presets_dir = get_example_presets_directory()
        if os.path.exists(presets_dir):
            preset_files = [f for f in os.listdir(presets_dir) if f.endswith('.hbat')]
            if preset_files:
                preset_name = preset_files[0].replace('.hbat', '')
                
                args = parser.parse_args([
                    "test.pdb", 
                    "--preset", preset_name,
                    "--hb-distance", "2.8"
                ])
                params = load_parameters_from_args(args)
                
                # Override should take effect
                assert params.hb_distance_cutoff == 2.8


class TestPresetManagement:
    """Test preset management functionality."""
    
    def test_preset_directory_access(self):
        """Test access to preset directory."""
        presets_dir = get_example_presets_directory()
        assert isinstance(presets_dir, str)
        
        # Directory might not exist in test environment
        if os.path.exists(presets_dir):
            assert os.path.isdir(presets_dir)
    
    def test_preset_listing(self, capsys):
        """Test preset listing functionality."""
        # This might not work in all test environments
        try:
            list_available_presets()
            captured = capsys.readouterr()
            
            # Should contain some output about presets
            assert "Available HBAT Presets" in captured.out or \
                   "No example presets directory found" in captured.out
        except Exception:
            # Acceptable if presets directory doesn't exist in test environment
            pass
    
    def test_preset_resolution(self):
        """Test preset path resolution."""
        presets_dir = get_example_presets_directory()
        
        if os.path.exists(presets_dir):
            preset_files = [f for f in os.listdir(presets_dir) if f.endswith('.hbat')]
            
            if preset_files:
                preset_name = preset_files[0].replace('.hbat', '')
                
                # Test resolving existing preset
                try:
                    resolved_path = resolve_preset_path(preset_name)
                    assert os.path.exists(resolved_path)
                    assert resolved_path.endswith('.hbat')
                except SystemExit:
                    # Acceptable if preset resolution fails in test environment
                    pass
    
    def test_preset_loading(self):
        """Test preset file loading."""
        presets_dir = get_example_presets_directory()
        
        if os.path.exists(presets_dir):
            preset_files = [f for f in os.listdir(presets_dir) if f.endswith('.hbat')]
            
            if preset_files:
                preset_path = os.path.join(presets_dir, preset_files[0])
                
                try:
                    params = load_preset_file(preset_path)
                    assert isinstance(params, AnalysisParameters)
                    assert params.hb_distance_cutoff > 0
                    assert params.hb_angle_cutoff > 0
                except (SystemExit, Exception):
                    # Acceptable if preset loading fails in test environment
                    pass
    
    def test_preset_file_format_validation(self):
        """Test preset file format validation."""
        # Create a temporary valid preset file
        valid_preset = {
            "format_version": "1.0",
            "application": "HBAT",
            "description": "Test preset",
            "parameters": {
                "hydrogen_bonds": {
                    "h_a_distance_cutoff": 3.5,
                    "dha_angle_cutoff": 120.0,
                    "d_a_distance_cutoff": 4.0
                },
                "halogen_bonds": {
                    "x_a_distance_cutoff": 4.0,
                    "cxa_angle_cutoff": 120.0
                },
                "pi_interactions": {
                    "h_pi_distance_cutoff": 4.5,
                    "dh_pi_angle_cutoff": 90.0
                },
                "general": {
                    "covalent_cutoff_factor": 1.2,
                    "analysis_mode": "complete"
                }
            }
        }
        
        with tempfile.NamedTemporaryFile(mode='w', suffix='.hbat', delete=False) as f:
            json.dump(valid_preset, f)
            temp_path = f.name
        
        try:
            params = load_preset_file(temp_path)
            assert isinstance(params, AnalysisParameters)
            assert params.hb_distance_cutoff == 3.5
            assert params.hb_angle_cutoff == 120.0
            assert params.analysis_mode == "complete"
        except (SystemExit, Exception):
            # Acceptable if loading fails in test environment
            pass
        finally:
            os.unlink(temp_path)


class TestCLIIntegration:
    """Test CLI integration with analysis engine."""
    
    @pytest.mark.integration
    def test_cli_parameter_to_analysis_integration(self, sample_pdb_file):
        """Test that CLI parameters properly configure analysis."""
        parser = create_parser()
        args = parser.parse_args([
            sample_pdb_file,
            "--hb-distance", "3.0",
            "--hb-angle", "140"
        ])
        
        params = load_parameters_from_args(args)
        
        # Verify parameters are correctly set
        assert params.hb_distance_cutoff == 3.0
        assert params.hb_angle_cutoff == 140.0
        
        # Test that these parameters can be used with analyzer
        from hbat.core.analysis import HBondAnalyzer
        analyzer = HBondAnalyzer(params)
        
        success = analyzer.analyze_file(sample_pdb_file)
        assert success, "Analysis with CLI parameters should succeed"
        
        # Verify analysis used the specified parameters
        assert analyzer.parameters.hb_distance_cutoff == 3.0
        assert analyzer.parameters.hb_angle_cutoff == 140.0
    
    def test_error_handling(self):
        """Test CLI error handling."""
        # Test invalid preset name
        try:
            resolve_preset_path("nonexistent_preset")
            assert False, "Should raise SystemExit for nonexistent preset"
        except SystemExit:
            # Expected behavior
            pass
        
        # Test invalid preset file
        with tempfile.NamedTemporaryFile(mode='w', suffix='.hbat', delete=False) as f:
            f.write("invalid json content")
            temp_path = f.name
        
        try:
            load_preset_file(temp_path)
            assert False, "Should raise SystemExit for invalid preset file"
        except (SystemExit, json.JSONDecodeError, Exception):
            # Expected behavior
            pass
        finally:
            os.unlink(temp_path)


class TestCLIHelp:
    """Test CLI help and documentation."""
    
    def test_help_contains_preset_options(self):
        """Test that help contains preset-related options."""
        parser = create_parser()
        help_text = parser.format_help()
        
        assert "--preset" in help_text
        assert "--list-presets" in help_text
        assert "Preset Options" in help_text
    
    def test_help_contains_examples(self):
        """Test that help contains usage examples."""
        parser = create_parser()
        help_text = parser.format_help()
        
        # Should contain examples section
        assert "Examples:" in help_text
        assert "--preset" in help_text  # Should show preset usage
    
    def test_parameter_help_descriptions(self):
        """Test that parameter help descriptions are informative."""
        parser = create_parser()
        help_text = parser.format_help()
        
        # Check that parameter descriptions mention units and defaults
        assert "Å" in help_text  # Distance units
        assert "degrees" in help_text  # Angle units
        assert "default:" in help_text.lower()  # Default values mentioned